<?php
/**
 * The original StripShow class
 * Prior to stripShow 3.0, comics were regular posts with a particular category. This is still
 * an option in 3.0, and if that option is enabled, then the $stripShow object will be of this
 * class rather than the StripShow class.
 *
 * @package stripShow
 * @author Brad Hawkins
 */
class StripShow_old {
	
	/**
	* A WP_Query object
	* @var WP_Query $comicQuery;
	* This class is intended to mimic the pre-3.0 functionality of stripShow,
	* wherein a comic post was a normal post in a particular category rather than
	* a custom post type.
	*/
	var $comicQuery;
	
	/**
	* A post object for a random comic.
	* This is randomized only once; all calls to this variable
	* will return the same object.
	* @var object $random_comic
	*/
	var $random_comic;
	
	/**
	* A post object for the current comic.
	* @var object $current_comic
	*/
	var $current_comic;
	
	/**
	* A post object for the first comic.
	* @var object $first_comic
	*/
	var $first_comic;
	
	/**
	* A post object for the previous comic.
	* @var object $previous_comic
	*/
	var $previous_comic;
	
	/**
	* A post object for the next comic.
	* @var object $next_comic
	*/
	var $next_comic;
	
	/**
	* A post object for the first comic.
	* @var object $last_comic
	*/
	var $last_comic;
	
	/**
	* Finds the parent of a specified story.
	* Finds the parent of a Storyline object and writes that information back to the object itself.
	* This function does not appear to be used, and I'm not entirely sure why it exists.
	* @param Storyline $story The Storyline object
	* @return object $story The same Storyline object with $parent variable set
	*/
	function find_story_parent( $story ) {
		if ( $story->parent ) {
			// This is a child story.
			$this->storylines[$story->parent]->children[$story->id] = $story;
			$story->level = $this->storylines[$story->parent]->level+1;
			return FALSE;
			}
		else {
			return $story;
			}
		}
	
	/**
	* Find the enddates of stories.
	* This function doesn't appear to be used.
	* @param array $stories An array of Storyline objects
	* @param string $parent_end The end date of the parent story
	*/
	function find_story_enddates( $stories, $parent_end='' ) {
		for ( $a = 0; $a < sizeof( $stories ); $a++ ) {
			$now = current( $stories );
			$next = next( $stories );
			$now->enddate=$next->startdate;
			if ( empty( $next->startdate ) && !empty( $parent_end ) ) $now->enddate = $parent_end;
			if ( ! empty( $now->children ) ) $this->find_story_enddates( $now->children, $now->enddate );
			}
		}

	function find_story_for_comic( $stories, $comic ) {
		foreach( $stories as $story ) {
			if ( ! empty( $story->children ) ) {
				$x = $this->find_story_for_comic( $story->children, $comic );
				if ( $x == 1 ) {
					if ( $story->level > 0 ) {
						return 1; // comic has been entered in a child, and this story is a child.
						}
					else { 
						break;
						}
					}
				}
			// no children had this comic in it, time to see whether it's in this story itself.
			$comicdate = $comic->post_date;
			$startdate = $story->startdate;
			$enddate = $story->enddate;
			if ( $comicdate >= $startdate ) {
				// Comic date must be on or after the start date.
				if ( $comicdate <= $enddate ) {
					//We have a hit!
					array_push( $story->parts, $comic );
					//echo "Found match.</div>";
					return 1;
					}
				elseif ( empty( $enddate ) ) {
					// this story has no end date
					array_push( $story->parts, $comic );

					//echo 'Found match.</div>';
					return 1;
					}
				}
			}
		//echo '</div>';
		return 0; // no matches found within this set of stories.
		}
	
	/**
	* Finds the current comic
	* Populates the $current_comic variable in this object
	* @uses $current_comic
	* @uses comicQuery
	*/
	function get_current_comic() {
		global $post, $wp_query, $wpdb;
		// If the current post is set, and it's a comic, then we simply put the $post object into $current_comic.
		if ( ! empty( $post ) ) {
			if ( $this->is_comic( $post ) && !is_home() ) {
				$this->current_comic = $post;
				$this->current_comic->permalink = 'hello';
				$this->comicQuery = $wp_query;
				return;
				}
			}
		$tempObj = $this->allComics;
		$indexgoesto = get_option( 'stripshow_indexgoesto' );
		switch( $indexgoesto ) {
			case 'first':
				$tempObj->set( 'order', 'ASC' );
				break;
			case 'random':
				$tempObj->set( 'orderby', 'rand' );
				$tempObj->set( 'order', '' );
				break;
			default:
				$tempObj->set( 'order', 'DESC' );
				break;
			}
			$tempObj->get_posts();

		$temp = $tempObj->posts;
		$this->current_comic = $temp[0];
		$this->current_comic->permalink = get_permalink( $temp[0]->ID );
		unset( $tempObj );
		}

	/**
	* Finds a comic or comics, whether by date or proximity to the current comic.
	* @param string $which A string of 'first', 'previous', 'next', 'last', 'random', or 'custom' to search by date.
	* @param int $howmany The number of comics to return
	* @param string $date The date for which to find comics, if finding by date
	* @return object $thepost The post object of the comic we find, or today's comic if no results were found
	* @uses stripshow_db_query
	* @uses stripshow_comic_category
	*/
	function get_comics_old( $which, $howmany=1, $date='' ) {
		global $wpdb;
		if ( empty( $this->current_comic ) ) return FALSE;
		$post = $this->current_comic;
		$current_date = $post->post_date;
		$multiple = FALSE;
		$limit = 1;
		switch ( $which ) {
			case 'first':
				$clause = "ORDER BY post_date ASC";
				break;
			case 'previous_few':
				$limit = $howmany;
			case 'previous':
				$clause = "AND post_date < '$current_date' ORDER BY post_date DESC";
				break;
			case 'next_few':
				$limit = $howmany;
			case 'next':
				$clause = "AND post_date > '$current_date' ORDER BY post_date ASC";
				break;
			case 'last':
				$clause = "ORDER BY post_date DESC";
				break;
			case 'random':
				$clause = "ORDER BY RAND()";
				break;
			case 'custom':
			default:
				if ( $date ) {
					$formatted_date_notime = date( 'Y-m-d 00:00:00', strtotime( $date ) );
					$formatted_date_withtime = date( 'Y-m-d H:i:s', strtotime( $date ) );
					if ( $formatted_date_withtime == $formatted_date_notime ) { //Date does not include time
						$clause = "AND ( DATE( post_date ) = '$formatted_date_notime' ) ORDER BY post_date DESC";
						}
					else { // date includes time
						$clause = "AND ( post_date = '$formatted_date_withtime' ) ORDER BY post_date DESC";
						}
					}
				else {
					return $this->current_comic;
					$clause = "AND ID = $post->ID";
					}
			}
		$querystring = stripshow_db_query() . $clause . " LIMIT $limit";
		//echo "<!-- $which: $querystring -->";
		if ( $limit == 1 ) $thepost = $wpdb->get_row( $querystring );
		else $thepost = $wpdb->get_results( $querystring );
		if ( $thepost ) {
			return $thepost; 
			}
		else return FALSE;
		}
	
	/**
	* Sets a WordPress query that includes only comics.
	*/
	function set_up_comics() {
		$querystring = '&cat='.stripshow_comic_category();
		global $wp_query;
		$allComics = new WP_Query();
   		$allComics->set( 'cat', stripshow_comic_category() );
		$allComics->set( 'orderby', 'post_date' );
		$allComics->set( 'order', 'DESC' );
		$allComics->set( 'post_status', 'publish' );
		$allComics->set( 'nopaging', 1 );
		$indexgoesto = get_option( 'stripshow_indexgoesto' );
		$wp_query->query_vars['category__not_in'] = $buffer_var;
		$allComics->get_posts();
		$bufferComics = wp_clone( $allComics );
		$bufferComics->set( 'post_status', 'future' );
		$bufferComics->get_posts();
		$this->allComics = $allComics;
		$this->bufferComics = $bufferComics;
		$this->comicQuery = new WP_Query( $querystring.'&showposts=1' );
		}

	function get_comics( $mode, $howmany=1, $date='' ) {
		$temp = wp_clone( $this->allComics );
		$temp->set( 'orderby', 'post_date' );
		switch( $mode ) {
			case 'first':
				$temp->set( 'order', 'ASC' );
				break;
			case 'last':
				$temp->set( 'order', 'DESC' );
				break;
			case 'next':
			case 'next_few':
				$temp->set( 'order', 'ASC' );
				add_filter( 'posts_where', 'filter_next', 1 );
				break;
			case 'previous':
			case  'previous_few':
				$temp->set( 'order', 'DESC' );
				add_filter( 'posts_where', 'filter_previous', 1 );
				break;
			case 'random':
				$temp->set( 'orderby', 'rand' );
				break;
			case 'custom':
			default:
				$temp->set( 'order', 'DESC' );
				if ( $date ) {
					$GLOBALS['stripshow_date_to_search_for'] = $date;
					add_filter( 'posts_where', 'filter_date', 1 );
					}
				else {
					$clause = "AND ID = $post->ID";
					}
				break;
			}
		$temp->set( 'nopaging', 0 );
		$temp->set( 'posts_per_page', $howmany );
		$temp->get_posts();
		$posts = $temp->posts;
		unset( $temp );
		remove_filter( 'posts_where', 'filter_next', 1 );
		remove_filter( 'posts_where', 'filter_previous', 1 );
		remove_filter( 'posts_where', 'filter_date', 1 );
		if ( sizeof( $posts ) > 1 ) return $posts;
		else return $posts[0];
		}

	function get_random_comic() {
		$original_query = $this->allComics->query;
		//echo $original_query;
		$this->allComics->query( $original_query . '&orderby=rand' );
		$first_temp = $this->allComics->posts;
		//echo '<pre>', var_dump( $this->allComics->posts ), '</pre>';
		$first_comic = $first_temp[0];
		return $first_comic;
		}
	
	/**
	* Checks whether a given post is a comic.
	*/
	function is_comic( $post = '' ) {
		global $wpdb, $stripShow;
		if ( ! empty( $post ) && check_comic_category( $post ) ) return TRUE;
		if ( in_the_loop() || is_single() ) {
			global $post;
			if ( check_comic_category( $post ) ) return TRUE;
			else return false;
			}
		if ( is_home() ) return TRUE;
	}

	function populate() {
		$this->get_current_comic();
		$GLOBALS['current_comic_date'] = $this->current_comic->post_date;
		$this->next_comic = $this->get_comics( 'next', 1 );
		$this->previous_comic = $this->get_comics( 'previous', 1 );
		$this->first_comic = $this->get_comics( 'first', 1 );
		$this->last_comic = $this->get_comics( 'last', 1 );
		$this->random_comic = $this->get_comics( 'random', 1 );
		$this->next_in_series = $this->get_comics( 'next_comic_in_series', 1 );
	}

	/**
	* Imports a single comic file into the archive.
	* This function creates a comic post and attaches a file to it.
	* @param string $filename The name of the file to import
	* @param int $timestamp The UNIX timestamp representing the date of the post
	* @param string $title A title to give the post
	* @param string $content Content to give the post
	* @since 2.5
	*/
	function import_one_comic($filename,$timestamp,$title='',$content='') {
		$content = replace_comic_wildcards($content,$timestamp,1);
		$title = replace_comic_wildcards($title,$timestamp,1);
		$current_date = date('Y-m-d H:i:s',$timestamp);
		$gmt_date = date('Y-m-d H:i:s',$timestamp+(get_option('gmt_offset')*-3600));
		if (current_user_can('publish_posts')) $status = 'publish'; else $status = 'pending';
			$myPost = array(
				'user_ID' => $GLOBALS['current_user']->ID, 
				'action' => 'post',
				'post_author' => $GLOBALS['current_user']->ID,
				'temp_ID' => -1139182474,
				'post_title' => $title,
				'post_pingback' => 1,
				'referredby' => 'redo',
				'advanced_view' => 1,
				'comment_status' => 'open',
				'ping_status' => 'open',
				'post_password' => '',
				'post_name' => sanitize_title($title),
				'post_category' => Array
					(
						'0' => get_option('stripshow_category')
					),

				'post_status' => $status,
				'post_content' => $content,
				'post_date_gmt' => $gmt_date,
				'tags_input' => $tags_input,
				'post_date' => $current_date );
			$post = wp_insert_post($myPost);
			if ($post) {
				if (!empty($_POST['stripshow_orientation'])) 
					add_post_meta($post,'orientation',$_POST['stripshow_orientation']);
				add_post_meta($post,'comic_file',$filename);
				$success = TRUE;
				return compact('success','post');
			}
			else {
				$success = FALSE;
				$error = "Could not add post to WordPress database.";
				return compact('success','error');
			}
	}

	/**
	* Constructor for the StripShow_old class.
	* This constructor sets up the current comic and populates
	* variables for the other comic links: first, previous, next, last,
	* and random.
	* @uses get_current_comic
	* @uses get_comics
	* @uses $random_comic
	* @uses $first_comic
	* @uses $next_comic
	* @uses $previous_comic
	* @uses $last_comic
	*/
	function StripShow_old() {
		//echo 'Constructing stripshow class';
        register_taxonomy( "character", array( "comic" ), array( 
            "hierarchical" => FALSE, 
            "label" => __( "Characters", 'stripshow' ), 
            "singular_label" => __( "Character", 'stripshow' ), 
            "rewrite" => true, 
            'query_var' => TRUE,
            'choose_from_most_used' => __( "Choose from Most Used Characters", 'stripshow' ),
            'labels' => array(
                'add_new_item' => __( "Add New Character", 'stripshow' ),
                'search_items' => __( 'Search Characters', 'stripshow' ),
                'all_items' => __( 'All Characters', 'stripshow' ),
                'separate_items_with_commas' => __( 'Separate character names with commas', 
					'stripshow' ),
                'choose_from_most_used' => __( 'Choose from the most used characters', 'stripshow' ),
                'popular_items' => __( 'Most-used Characters', 'stripshow' ),
                )
            )
        );
		$this->set_up_comics();
	}
}