<?php


/**
* Represents one comic archive
* @package stripShow
*/
class StripShow {
	
	/**
	* A WP_Query object
	* @var WP_Query $comicQuery;
	*/
	var $comicQuery;
	
	/**
	* A post object for a random comic.
	* This is randomized only once; all calls to this variable
	* will return the same object.
	* @var object $random_comic
	*/
	var $random_comic;
	
	/**
	* A post object for the current comic.
	* @var object $current_comic
	*/
	var $current_comic;
	
	/**
	* A post object for the first comic.
	* @var object $first_comic
	*/
	var $first_comic;
	
	/**
	* A post object for the previous comic.
	* @var object $previous_comic
	*/
	var $previous_comic;
	
	/**
	* A post object for the next comic.
	* @var object $next_comic
	*/
	var $next_comic;
	
	/**
	* A post object for the first comic.
	* @var object $last_comic
	*/
	var $last_comic;
	
	/**
	* Finds the parent of a specified story.
	* Finds the parent of a Storyline object and writes that information back to the object itself.
	* This function does not appear to be used, and I'm not entirely sure why it exists.
	* @param Storyline $story The Storyline object
	* @return object $story The same Storyline object with $parent variable set
	*/
	function find_story_parent( $story ) {
		if ( $story->parent ) {
			// This is a child story.
			$this->storylines[$story->parent]->children[$story->id] = $story;
			$story->level = $this->storylines[$story->parent]->level+1;
			return FALSE;
		}
		else {
			return $story;
		}
	}
	
	/**
	* Find the enddates of stories.
	* This function doesn't appear to be used.
	* @param array $stories An array of Storyline objects
	* @param string $parent_end The end date of the parent story
	*/
	function find_story_enddates( $stories, $parent_end='' ) {
		for ( $a = 0; $a < sizeof( $stories ); $a++ ) {
			$now = current( $stories );
			$next = next( $stories );
			$now->enddate=$next->startdate;
			if ( empty( $next->startdate ) && !empty( $parent_end ) ) $now->enddate = $parent_end;
			if ( ! empty( $now->children ) ) $this->find_story_enddates( $now->children, $now->enddate );
		}
	}

/**
 * Finds which story contains a given comic.
 *
 * @param array $stories 
 * @param string $comic 
 * @return void
 * @author Brad Hawkins
 */
	function find_story_for_comic( array $stories, $comic ) {
		foreach( $stories as $story ) {
			if ( ! empty( $story->children ) ) {
				$x = $this->find_story_for_comic( $story->children, $comic );
				if ( $x == 1 ) {
					if ( $story->level > 0 ) {
						// comic has been entered in a child, and this story is a child.
						return 1; 
					}
					else { 
						break;
					}
				}
			}
			// no children had this comic in it, time to see whether it's in this story itself.
			$comicdate = $comic->post_date;
			$startdate = $story->startdate;
			$enddate = $story->enddate;
			if ( $comicdate >= $startdate ) {
				// Comic date must be on or after the start date.
				if ( $comicdate <= $enddate ) {
					//We have a hit!
					array_push( $story->parts, $comic );
					//echo "Found match.</div>";
					return 1;
				} elseif ( empty( $enddate ) ) {
					// this story has no end date
					array_push( $story->parts, $comic );

					//echo 'Found match.</div>';
					return 1;
				}
			}
		}
		//echo '</div>';
		return 0; // no matches found within this set of stories.
	}
	
	/**
	* Finds which comic is current.
	* Populates the $current_comic variable in this object. "Current" in this context means
	* A) The comic being viewed, if viewing a single post.
	* B) The latest comic, if not.
	* @uses $current_comic
	* @uses comicQuery
	*/
	function get_current_comic() {
		global $post, $wp_query, $wpdb;
		// If the current post is set, and it's a comic, then we simply put the $post object into $current_comic.
		if ( ! empty( $post ) ) {
			if ( is_comic( $post ) && !is_home() ) {
				$this->current_comic = $post;
				$this->current_comic->permalink = get_permalink( $post->ID );
				$this->comicQuery = $wp_query;
				return;
			}
		}
		$tempObj = new WP_Query( 'showposts=1&post_type=comic' );
		$indexgoesto = get_option( 'stripshow_indexgoesto' );
		switch( $indexgoesto ) {
		case 'first':
			$tempObj->set( 'order', 'ASC' );
			break;
		case 'random':
			$tempObj->set( 'orderby', 'rand' );
			$tempObj->set( 'order', '' );
			break;
		default:
			$tempObj->set( 'order', 'DESC' );
			break;
		}
		$tempObj->get_posts();

		$temp = $tempObj->posts;
		$this->current_comic = $temp[0];
		$this->current_comic->permalink = get_permalink( $temp[0]->ID );
		unset( $tempObj );
	}

	
	/**
	* Sets a WordPress query that includes only comics.
	*/
	function set_up_comics() {
		$querystring = '&post_type=comic';
		global $wp_query;
		$allComics = new WP_Query();
		$allComics->set( 'post_type', 'comic' );
		$allComics->set( 'orderby', 'post_date' );
		$allComics->set( 'order', 'DESC' );
		$allComics->set( 'post_status', 'publish' );
		$allComics->set( 'nopaging', 1 );
		$allComics->set( 'cache_results', 0 );
		$indexgoesto = get_option( 'stripshow_indexgoesto' );
		$wp_query->query_vars['category__not_in'] = $buffer_var;
		$allComics->get_posts();
		$bufferComics = wp_clone( $allComics );
		$bufferComics->set( 'post_status', 'future' );
		$bufferComics->get_posts();
		$this->allComics = $allComics;
		$this->bufferComics = $bufferComics;
		//$this->comicQuery = new WP_Query( $querystring.'&suppress_filters=1&posts_per_page=1' );
		$this->comicQuery = wp_clone( $this->allComics );
		$this->comicQuery->set( 'showposts', 1 );
		$this->comicQuery->set( 'nopaging', 0 );
		$this->comicQuery->get_posts();
	}

	function get_comics( $mode, $howmany=1, $date='' ) {
	    $debug = false;
		$temp = wp_clone( $this->allComics );
		$temp->set( 'orderby', 'post_date' );
		switch( $mode ) {
		case 'first':
			$temp->set( 'order', 'ASC' );
			break;
		case 'last':
			$temp->set( 'order', 'DESC' );
			break;
		case 'next':
		case 'next_few':
			$temp->set( 'order', 'ASC' );
			add_filter( 'posts_where', 'filter_next', 1 );
			break;
		case 'next_in_series':
			$temp->set( 'order', 'ASC' );
			$temp->set( 'comic-series', 'test-series' );
			add_filter( 'posts_where', 'filter_next', 1 );
			break;
		case 'previous':
		case  'previous_few':
			$temp->set( 'order', 'DESC' );
			add_filter( 'posts_where', 'filter_previous', 1 );
			break;
		case 'random':
			$temp->set( 'orderby', 'rand' );
			break;
		case 'custom':
		default:
			$temp->set( 'order', 'DESC' );
			if ( $date ) {
				$GLOBALS['stripshow_date_to_search_for'] = $date;
				add_filter( 'posts_where', 'filter_date', 1 );
			} else {
				$clause = "AND ID = $post->ID";
			}
			break;
		}
		$temp->set( 'nopaging', 0 );
		$temp->set( 'posts_per_page', $howmany );
		$temp->get_posts();
		if ( $debug ) echo $temp->request;
		$posts = $temp->posts;
		unset( $temp );
		remove_filter( 'posts_where', 'filter_next', 1 );
		//remove_filter( 'posts_where', 'filter_next_in_series', 1 );
		remove_filter( 'posts_where', 'filter_previous', 1 );
		remove_filter( 'posts_where', 'filter_date', 1 );
		if ( sizeof( $posts ) > 1 ) return $posts;
		if ( $debug ) echo '<pre>'.$mode.':'.$posts[0]->post_title.'</pre>';
		else return $posts[0];
		}

	function get_random_comic() {
		$original_query = $this->allComics->query;
		//echo $original_query;
		$this->allComics->query( $original_query . '&orderby=rand' );
		$first_temp = $this->allComics->posts;
		//echo '<pre>', var_dump( $this->allComics->posts ), '</pre>';
		$first_comic = $first_temp[0];
		return $first_comic;
		}
	
    function add_comic_type() {
        register_post_type( 'comic', array(
            'label' => __( 'Comics', 'stripshow' ),
            'singular_label' => 'Comic',
            'public' => TRUE,
            'show_ui' => TRUE,
            'show_in_menu' => TRUE,
            'capability_type' => 'post',
            'heirarchical' => FALSE,
            'rewrite' => array( 'slug' => 'comic' ),
            'query_var' => FALSE,
            'register_meta_box_cb' => array( &$this, 'stripshow_meta_box' ),
            'supports' => array( 'title', 'editor', 'author', 'comments', 'categories', 'role-scoper' ),
            'builtin' => FALSE,
            'menu_icon' => WP_PLUGIN_URL . '/' . STRIPSHOW_PLUGIN_DIR . '/admin/images/stripshow-icon-small.gif',
            'taxonomies' => array( 'character', 'comic-series' ),
            'labels' => array(
                'add_new_item' => __( 'Add New Comic', 'stripshow' ),
                'add_new' => __( 'Add New' ),
                'search_items' => __( 'Search Comics', 'stripshow' ),
                'not_found' => __( 'No comics found', 'stripshow' ),
                'not_found_in_trash' => __( 'No comics found in trash', 'stripshow' ),
                )
            )
        );
        register_taxonomy( "character", array( "comic" ), array( 
            "hierarchical" => FALSE, 
            "label" => __( "Characters", 'stripshow' ), 
            "singular_label" => __( "Character", 'stripshow' ), 
            "rewrite" => true, 
            'query_var' => 'character',
            'choose_from_most_used' => __( "Choose from Most Used Characters", 'stripshow' ),
            'labels' => array(
                'add_new_item' => __( "Add New Character", 'stripshow' ),
                'search_items' => __( 'Search Characters', 'stripshow' ),
                'all_items' => __( 'All Characters', 'stripshow' ),
                'separate_items_with_commas' => __( 'Separate character names with commas', 'stripshow' ),
                'choose_from_most_used' => __( 'Choose from the most used characters', 'stripshow' ),
                'popular_items' => __( 'Most-used Characters', 'stripshow' ),
                )
            )
        );
        register_taxonomy( "comic-series", array( "comic" ), array( 
            "hierarchical" => true, 
            "label" => __( "Series", 'stripshow' ), 
            "singular_label" => __( "Series", 'stripshow' ), 
            "rewrite" => true, 
            'query_var' => 'comic-series',
            'labels' => array(
                'add_new_item' => __( "Add New Series", 'stripshow' ),
                'search_items' => __( 'Search Series', 'stripshow' ),
                'all_items' => __( 'All Series', 'stripshow' ),
                'separate_items_with_commas' => __( 'Separate series with commas', 'stripshow' ),
                )
            
            ) 
        );
        add_action( "template_redirect", array( &$this, 'template_redirect' ) );

    }    

	// Template selection
	function template_redirect()
	{
		global $wp, $wp_query;
		if ( $wp->query_vars["post_type"] == "comic" && !is_feed() )
		{
		    if ( have_posts() ) {
                if ( file_exists ( TEMPLATEPATH . "/single-comic.php" ) ) 			include( TEMPLATEPATH . "/single-comic.php" );
                else include( TEMPLATEPATH. "/single.php" );
                die();
            }
            else {
               $wp_query->is_404 = true;
            }
		}
	}

function stripshow_meta_box() {
	add_meta_box( "stripshow_post_meta", __( 'Comic Information', 'stripshow' ), "stripshow_post_meta_box", "comic", 'normal', 'high' );
}
	
	/**
	* Fills this object with current, first, previous, next, last, and random
	* comics.
	* This must be run after the post is created, which is why it is loaded
	* as part of template_redirect rather than init.
	*/
	function populate() {
		$this->get_current_comic();
		$GLOBALS['current_comic_date'] = $this->current_comic->post_date;
		$this->next_comic = $this->get_comics( 'next', 1 );
		$this->previous_comic = $this->get_comics( 'previous', 1 );
		$this->first_comic = $this->get_comics( 'first', 1 );
		$this->last_comic = $this->get_comics( 'last', 1 );
		$this->random_comic = $this->get_comics( 'random', 1 );
	}

	function is_comic( $post='' ) {
		global $wpdb, $stripShow, $post;
		if ( sizeof( $this->allComics->posts ) < 1 ) return FALSE; // Are there any comics at all?
		if ( is_singular( 'comic' ) ) return TRUE;
		if ( is_home() ) return TRUE;
		if ( ! empty( $post ) ) {
			if ( $post->post_type == 'comic' ) return TRUE;
			else return FALSE;
		}            
	}
	
	/**
	* Imports a single comic file into the archive.
	* This function creates a comic post and attaches a file to it.
	* @param string $filename The name of the file to import
	* @param int $timestamp The UNIX timestamp representing the date of the post
	* @param string $title A title to give the post
	* @param string $content Content to give the post
	* @since 2.5
	*/
	function import_one_comic($filename,$timestamp,$title='',$content='') {
		$content = replace_comic_wildcards($content,$timestamp,1);
		$title = replace_comic_wildcards($title,$timestamp,1);
		$current_date = date('Y-m-d H:i:s',$timestamp);
		$gmt_date = date('Y-m-d H:i:s',$timestamp+(get_option('gmt_offset')*-3600));
		if (current_user_can('publish_posts')) $status = 'publish'; else $status = 'pending';
		$myPost = array(
			'user_ID' => $GLOBALS['current_user']->ID, 
			'action' => 'post',
			'post_author' => $GLOBALS['current_user']->ID,
			'temp_ID' => -1139182474,
			'post_title' => $title,
			'post_pingback' => 1,
			'referredby' => 'redo',
			'advanced_view' => 1,
			'comment_status' => 'open',
			'ping_status' => 'open',
			'post_type' => 'comic',
			'post_password' => '',
			'post_name' => sanitize_title($title),

			'post_status' => $status,
			'post_content' => $content,
			'post_date_gmt' => $gmt_date,
			'tags_input' => $tags_input,
			'post_date' => $current_date );
		//var_dump($myPost);
		$post = wp_insert_post($myPost);
		//echo $post;
		if ($post) {
			if (!empty($_POST['stripshow_orientation'])) add_post_meta($post,'orientation',$_POST['stripshow_orientation']);
			add_post_meta($post,'comic_file',$filename);
			$success = TRUE;
			return compact('success','post');
		}
		else {
			$success = FALSE;
			$error = "Could not add post to WordPress database.";
			return compact('success','error');
		}
	}

	/**
	* Constructor for the StripShow class.
	* This constructor sets up the current comic and populates
	* variables for the other comic links: first, previous, next, last,
	* and random.
	* @uses get_current_comic
	* @uses get_comics
	* @uses $random_comic
	* @uses $first_comic
	* @uses $next_comic
	* @uses $previous_comic
	* @uses $last_comic
	*/
	function StripShow() {
		$this->add_comic_type();
		$this->set_up_comics();
		}
	}
	
function filter_next( $clause='' ) {
	global $wpdb;
	$current_date = $GLOBALS['current_comic_date'];
	$clause .= " AND $wpdb->posts.post_date > '$current_date'";
	//echo $clause;
	return $clause;
	}
function filter_previous( $clause='' ) {
	global $wpdb;
	$current_date = $GLOBALS['current_comic_date'];
	$clause .= " AND $wpdb->posts.post_date < '$current_date'";
	//echo $clause;
	return $clause;
	}
function filter_date( $clause='' ) {
	$date = $GLOBALS['stripshow_date_to_search_for'];
	global $wpdb;
	//echo $date;
	$formatted_date_notime = date( 'Y-m-d 00:00:00', strtotime( $date ) );
	$formatted_date_withtime = date( 'Y-m-d H:i:s', strtotime( $date ) );
	if ( $formatted_date_withtime == $formatted_date_notime ) { //Date does not include time
		$clause .= "AND ( DATE( $wpdb->posts.post_date ) = '$formatted_date_notime' )";
		}
	else { // date includes time
		$clause .= "AND ( $wpdb->posts.post_date = '$formatted_date_withtime' )";
		}

	//echo $clause;
	return $clause;
	}

	
