<?php
/**
* stripShow installation routine.
* Runs whenever the stripshow_version option is not set.
* Seeks out ComicPress settings (if a ComicPress theme is active)
* and converts them to stripShow settings.
* @uses storylines_install
* @uses stripshow_recurse_copy
* @uses stripshow_autocomic_defaults
*/
function stripshow_install() {
	$stripshow_notices = array();
	$stripshow_errors = array();
	$stripshow_notices[] = __( 'Installing stripShow...', 'stripshow' );
	
	// Determine whether there are already ComicPress settings
	$comicpress_config = get_template_directory() . '/comicpress-config.php';
	if ( file_exists( $comicpress_config ) ) {
		$has_comicpress = TRUE;
		$comicpress = file( $comicpress_config, FILE_SKIP_EMPTY_LINES); // read in the entire comicpress-config.php file -- for some reason, just including it doesn't work -- to get all the variables.
		foreach ( $comicpress as $line ) {
			$line = str_replace( "<"."?php", '', $line );
			$line = str_replace( "?".">", '', $line );
			eval( $line );
		}
		$stripshow_notices[] = __( 'Found and imported ComicPress settings.','stripshow' );
	}
	else 
		$has_comicpress = FALSE;

	if ( $has_comicpress ) {
		$date_format = 'Y-m-d';
		$catstyle = 'comicpress';
		$rss_setting = 0;
	}
	else {
		$comic_folder = 'wp-content/stripshow_comics';
		$rss_comic_folder = 'wp-content/stripshow_comics';
		$archive_comic_folder = 'wp-content/stripshow_comics';
		$thumbnail_folder = 'wp-content/stripshow_comics/thumbnails';
		$special_folder = 'wp-content/stripshow_comics/special';
		$blogcat = 1;
		$date_format = 'Ymd';
		$catstyle = 'stripshow';
		$rss_setting = 1;
	}
	update_option ( 'stripshow_version', STRIPSHOW_VERSION );
	add_option ( 'stripshow_comicdir', $comic_folder, '', 'yes' );
	add_option ( 'stripshow_rssdir', $rss_comic_folder, '', 'yes' );
	add_option ( 'stripshow_archivedir', $archive_comic_folder, '', 'yes' );
	add_option ( 'stripshow_date_format', $date_format, '', 'yes' );
	add_option ( 'stripshow_catstyle', $catstyle, '', 'yes' );
	add_option( 'stripshow_indexgoesto', 'last', '', 'yes' );
	add_option( 'stripshow_walkback', 1, '' );
	add_option( 'stripshow_rss_comics', $rss_setting, '' );
	add_option( 'stripshow_hide_comic_path', 0, '' );
	add_option( 'stripshow_thumbnail_style', 1, '' );
	add_option( 'stripshow_thumbnail_percent', 50, '' );
	add_option( 'stripshow_thumbnail_max_width', 100, '' );
	add_option( 'stripshow_thumbnail_max_height', 100, '' );
	add_option( 'stripshow_flash_code', '<object type="application/x-shockwave-flash" width="%width" height="%height" id="myMovieName" data="%file"><param name="movie" value="%url" /><param name="loop" value="false" /></object>' );
	
	stripshow_autocomic_defaults();
	
	// Set up categories if user hasn't already done this.
	$numcats = get_all_category_ids();
	if ( sizeof( $numcats ) == 1) { // There's only the "uncategorized" category
		$cat_name = __( 'Comics', 'stripshow' );
		$category_description = __( 'My Comics', 'stripshow' );
		$category_nicename= "comics";
		$cat_array = compact( 'cat_name', 'category_description', 'category_nicename' );
		$comiccat = wp_insert_category( $cat_array );
		$stripshow_notices[] = __( 'Added comic category.', 'stripshow' );
	}
	if ( empty( $comiccat ) ) 
		$comiccat = 3;
	add_option ( 'stripshow_category', $comiccat, '', 'yes' );
	add_option ( 'stripshow_nonstrip_category', $blogcat, '', 'no' );
	
	//Attempt to set up comics folder
	$comicpath = ABSPATH . ltrim( get_option( 'stripshow_comicdir' ), '/' );
	if ( ! file_exists( $comicpath ) ) {
		mkdir( $comicpath );
		if ( file_exists( $comicpath ) ) {
			$stripshow_notices[] = sprintf( __( 'Created comics folder at %s', 'stripshow' ), $comicpath );
			}
		else $stripshow_errors[] = sprintf( __( 'Could not create comics folder at %s', 'stripshow' ), $comicpath );
	}
	
	//Attempt to set up thumbnails folder
	$thumbpath = ABSPATH . ltrim( get_option( 'stripshow_thumbnaildir' ), '/' );
	if ( !file_exists( $thumbpath ) ) {
		mkdir( $thumbpath );
		if ( file_exists( $thumbpath ) ) {
			$stripshow_notices[] = sprintf( __( 'Created thumbnails folder at %s', 'stripshow' ), $thumbpath );
			}
		else $stripshow_errors[] = sprintf( __( 'Could not create thumbnails folder at %s', 'stripshow' ), $thumbpath );
	}
	
	//Attempt to set up special comics folder
	$specialpath = ABSPATH . ltrim( get_option( 'stripshow_specialdir' ), '/' );
	if ( !file_exists( $specialpath ) ) {
		mkdir( $specialpath );
		if ( file_exists( $specialpath ) ) {
			$stripshow_notices[] = sprintf( __( 'Created special comics folder at %s', 'stripshow' ), $specialpath );
			}
		else $stripshow_errors[] = sprintf( __( 'Could not create special comics folder at %s', 'stripshow' ), $specialpath );
	}
	
	
	// Attempt to create symlink to stripshow_sandbox theme in themes dir
	$location = preg_replace( '/[^\/]*$/', '', TEMPLATEPATH );
	if ( ! file_exists( $location.'stripshow_sandbox' ) ) {
		$source = '../plugins/stripshow/example-themes/stripshow_sandbox';
		$dest = $location . 'stripshow_sandbox';
		if ( file_exists( $dest ) ) {
			$stripshow_notices[] = __( 'A file named stripshow_sandbox already exists in your themes folder', 'stripshow' );
		}
		else if ( symlink( $source, $dest ) ) 
			$stripshow_notices[] = __( 'Created symbolic link to stripShow Sandbox theme in your themes directory.', 'stripshow' );
		else 
			$stripshow_errors[] = __( 'Could not create symbolic link to stripShow Sandbox theme in your themes directory.', 'stripshow' );
	}
		
	// Attempt to copy the stripshow_child theme to the themes dir.
	$themes_dir = dirname( TEMPLATEPATH );
	if ( ! file_exists( $themes_dir . '/stripshow_child' ) ) {
		$copy_success = stripshow_recurse_copy ( WP_PLUGIN_DIR . '/' . STRIPSHOW_PLUGIN_DIR . '/example-themes/stripshow_child', $themes_dir . '/stripshow_child' );
		if ( $copy_success ) $stripshow_notices[] = __( 'Copied stripShow Sandbox Child Theme to themes folder.', 'stripshow' );
		else $stripshow_errors[] = __( 'Could not copy stripShow Sandbox Child Theme to themes folder.', 'stripshow' ); 
	}

	storylines_install();
	$stripshow_notices[] = __( 'stripShow has been successfully installed.', 'stripshow' )	;
	update_option( 'stripshow_errors', array( $stripshow_notices, $stripshow_errors ) );
}
/*
* Recursively copy one directory to another.
* This function is used by the stripshow_install function to copy the
* stripshow_child folder into themes directory.
* This function was lifted from user contributions to the PHP online manual.
* @param string $src The file to copy
* @param string $dst The destination
*/
function stripshow_recurse_copy( $src, $dst ) {
	$dir = opendir( $src );
	if ( ! $dir ) 
		return FALSE;
	$mkdir_success = @mkdir( $dst );
	if ( ! $mkdir_success )
		return FALSE;
	while( false !== ( $file = readdir( $dir ) ) ) {
		if ( ( $file != '.' ) && ( $file != '..' ) ) {
			if ( is_dir( $src . '/' . $file ) ) {
				stripshow_recurse_copy( $src . '/' . $file, $dst . '/' . $file );
			}
			else {
				$success = @copy( $src . '/' . $file, $dst . '/' . $file );
				if ( ! $success ) 
					return FALSE;
			}
		}
	}
	closedir( $dir );
	return TRUE;			
}

/**
 * Performs the tasks to upgrade from a prior version.
 * At present, it only checks to see if the prior version was pre-3.0.
 * If additional database changes are instituted in later versions,
 * the logic to test for those differences will be here as well.
 *
 * @param string $old_version 
 * @return void
 * @author Brad Hawkins
 */
function stripshow_upgrade( $old_version ) {
	if ( version_compare( $old_version, '3.0', '<' ) ) {
		$report = 'Upgrading from a version prior to 3.0.';
		update_option( 'stripshow_version', STRIPSHOW_VERSION );
		update_option( 'stripshow_upgrade', $old_version );
		wp_redirect( '?page=stripshow-menu');
	} else {
		$report = 'Upgrading from a version after 3.0.';
	}
	update_option( 'stripshow_errors', $report );
}

/**
* Creates storyline table.
* Called at plugin activation, creates a MySQL table for storylines in 
* WP database if none exists.
*/
function storylines_install() {
	global $wpdb;
	$table_name = $wpdb->prefix . "storylines";
	if( $wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) != $table_name ) {
		// no existing storylines table
		$sql = "CREATE TABLE " . $table_name . " (
			id mediumint(9) NOT NULL AUTO_INCREMENT,
			name tinytext NOT NULL,
			startdate date NOT NULL,
			enddate date,
			parent mediumint(9),
			UNIQUE KEY id (id)
			);";
	} 
	else {
		// Check to see if the enddate colum is set to NOT NULL -- this was a bug in earlier versions of stripShow -- and fix it if it is
		$tableinfo = $wpdb->get_results( "DESCRIBE $table_name enddate;", ARRAY_A );
		if ( $tableinfo[0]['Null'] = 'NO' ) {
			$x = $wpdb->query( "ALTER TABLE $table_name MODIFY COLUMN enddate DATE NULL;" );
		}
	}
	require_once( ABSPATH . 'wp-admin/upgrade-functions.php' );
	dbDelta( $sql );
}

function stripshow_upgrade_box( $previous_version ) {
    $nonce = wp_create_nonce('stripshow_ajax_upgrade_nonce'); 
	?>
	  <script type="text/javascript">
	    var nonce = '<?php echo $nonce; ?>';
	  </script>
	<div id="stripshow-upgrade-box">
		<?php printf( __( 'You have upgraded from stripShow %s', 'stripshow' ), $previous_version ); ?>
		<p id="upgrade-status">
			<?php _e( 'Do you want to convert your comics from regular posts to the new <strong>Comic</strong> post type?', 
				'stripshow') ?><br/><br/>
			<input type="button" name="upgrade-comics-yes" id="upgrade-comics-yes" value="Yes"/>&nbsp;
			<input type="button" name="upgrade-comics-no" id="upgrade-comics-no" value="No"/>
		</p>
	</div>
	<?php
	
}

function stripshow_ajax_upgrade_comics() {
	global $wpdb, $stripShow; 
	update_option( 'stripshow_post_type', 'comic');
	delete_option( 'stripshow_upgrade' );
	if ( ! check_ajax_referer( 'stripshow_ajax_upgrade_nonce' ) )
		die( __( 'Security breach', 'stripshow' ) );
	$category = stripshow_comic_category();
	$temp = new WP_Query( 'nopaging=TRUE&cat='.$category );
	$query = $temp->request;
	$result = $wpdb->get_results( $query );
	$x = 0;
	if ( empty($result) ) return;
	foreach ( $result as $post ) {
		$newpost = array();
		$newpost['ID'] = $post->ID;
		$newpost['post_type'] = 'comic';
		$success = wp_update_post( $newpost );
		if ( $success ) $x++;
		unset($newpost);
	}
	echo $x;
	die();
}

function stripshow_upgrade_old_posts( $cat ) {
    global $wpdb;
    $temp = new WP_Query( 'nopaging=TRUE&cat='.$cat );
    $query = $temp->request;
    $result = $wpdb->get_results( $query );
	$x = 0;
    if ( empty($result) ) return;
    foreach ( $result as $post ) {
        $newpost = array();
        $newpost['ID'] = $post->ID;
        $newpost['post_type'] = 'comic';
        //$success = wp_update_post( $newpost );
        $output .= sprintf( __( 'Upgraded %s.', 'stripshow' ), $post->post_title );
        $output .= '<br/>';
		wp_ob_end_flush_all();
        flush();
        unset($newpost);
    }
	return $output;
}
